clear all;
[a, fs] = wavread('C:\Users\jl\Desktop\test.wav');

%% lange Faltung
n = 2^18; % Lnge von Wellenform und Impulsantwort, Zweierpotenz fr FFT
z = zeros(n, 1);
d = 0.02*fs; % Millisekunden -> Samples 
% Impulsantwort = ein Delta-Puls, dann d Sekunden Stille,
% dann exponentiell abklingendes, Gausches Rauschen
impulseResponse = [200.0; zeros(d, 1); randn(n-d-1, 1) .* exp(-(1:n-d-1)*0.00005)'];
sound(impulseResponse/200.0, fs, 16);
s1 = [a(1:n, 1); z]; % Padding mit Nullen, damit der Hall des Endes nicht den Anfang berlagert
s2 = [impulseResponse; z];
f1 = fft(s1);
f2 = fft(s2);
s3 = real(ifft(f1.*f2));
soundsc(s3, fs, 16);

%% Hilbert-Transformation
z = hilbert(a(:, 1));
plot((1:length(a))/fs, real(z), (1:length(a))/fs, imag(z));
% 90 Phasenverschiebung zwischen links und rechts
sound([real(z), imag(z)], fs, 16);
% fiese Frequenzverschiebung, besonders gut in den Bssen zu hren
q = exp(100i/fs*(1:length(a)))' .* z;
sound(real(q), fs, 16);

%% LPC-Filterung
% Achtung, wenn MIRtoolbox installiert:
% Erst Auditory Toolbox aus dem Pfad nehmen.
orig = wavread('C:\Users\jl\Desktop\ooo.wav');
view(90, -90);

ai = lpc(orig(:, 1), 50);
bi = zeros(1, length(ai));
bi(1) = 1;
filtered = filter(bi, ai, a(:, 1));

spectrogram([orig(:, 1); filtered], 4096, 16, 4096, fs);
view(90, -90);

soundsc(filtered, fs, 16);

%% Physical Modeling: gezupfte Saite (Karplus-Strong)
n = int32(fs / 110.0);
b = randn(n, 1);
numSamples = 3 * fs;
c = zeros(numSamples, 1);
for i = 1 : numSamples
    c(i) = b(mod(i, n) + 1);
    b(mod(i, n) + 1) = 0.5*(b(mod(i, n) + 1) + b(mod(i+1, n) + 1));
end
plot((1:length(c))/fs, c);
soundsc(c, fs, 16);

%% Batchverarbeitung
% Nehme alle .wav-Dateien auf dem Desktop
% und kopiere sie rckwrts abgespielt
% in das Verzeichnis "converted".
path = 'C:\Users\jl\Desktop';
files = dir(path);
for i = 1:length(files)
    if files(i).isdir
        continue
    end
    f = files(i).name;
    if ~strcmpi(f(end-3:end),'.wav')
        continue
    end
    auIn = wavread([path, '\', f]);
    auOut = auIn(end:-1:1, 1); % linker Kanal rckwrts
    wavwrite(auOut, fs, 16, [path, '\converted\', f]);
end

%% MIRtoolbox einfach
% MIRtoolbox herunterladen von
% https://www.jyu.fi/hum/laitokset/musiikki/en/research/coe/materials/mirtoolbox
au = miraudio('C:/Users/jl/Desktop/test_mir.wav');
f = mirframe(au, 'Length', 2, 's');
mirchromagram(f) % kein Semikolon: Ausgabe!
mirkeystrength(f) % kein Semikolon: Ausgabe!
mirtempo(f) % kein Semikolon: Ausgabe!

%% MIRtoolbox im Batchbetrieb
% Klassifikation von Musikgenres ber den Klang;
% Die Dateien in TracksTraining und TracksTest
% haben als ersten Buchstaben im Dateinamen
% ein Zeichen fr ihres Genres (per Hand),
% z.B. c = classic, j = jazz, p = pop, r = rock
cd C:/Users/jl/Desktop/TracksTraining
training = miraudio('Folder', 'Label', 1:1);
cd ../TracksTest
test = miraudio('Folder', 'Label', 1:1);
cd ..
mfccTraining = mirmfcc(training);
mfccTest = mirmfcc(test);
mirclassify(test, mfccTest, training, mfccTraining)

pause

mirclassify(test, mfccTest, training, mfccTraining, 5)

pause

mirclassify(test, mfccTest, training, mfccTraining, 'GMM', 3)

%% In der Wellenform den Playhead mitfhren
plot((1:length(a))/fs, a(:, 1));
plr = audioplayer(a, fs);
total = get(plr, 'TotalSamples');
play(plr); % asynchron, notfalls mit stop (plr); beenden
lineHandle = line([0, 0], [-1, 1], [0, 0], 'Color','r','LineWidth', 2);
while(strcmp(get(plr,'Running'), 'on'))
    x = get(plr, 'CurrentSample') / total;
    set(lineHandle, 'XData', [x, x]*length(a)/fs);
    pause(0.1);
end

%% GUI-Editor
guide